#include <core/type_details.hpp>

namespace core {

std::unique_ptr < type_details_base > type_details_retriever < void >::get ( ) {
	return std::make_unique < type_details_type > ( "void" );
}

type_details::type_details ( std::unique_ptr < type_details_base > data ) : m_type_details ( std::move ( data ) ) {
}

type_details type_details::universal_type ( ) {
	return type_details ( std::make_unique < core::type_details_universal_type > ( ) );
}

type_details type_details::void_type ( ) {
	return type_details ( type_details_retriever < void >::get ( ) );
}

type_details type_details::as_type ( const std::string & string ) {
	return type_details ( std::make_unique < type_details_type > ( string ) );
}

type_details type_details::as_type ( const std::string & string, const std::vector < std::string > & templateParams ) {
	std::vector < std::unique_ptr < type_details_base > > templates;
	for ( const std::string & templateParam : templateParams ) {
		templates.push_back ( std::make_unique < type_details_type > ( templateParam ) );
	}
	return type_details ( std::make_unique < type_details_template > ( string, std::move ( templates ) ) );
}

bool type_details::compatible_with ( const type_details & other ) const {
	return m_type_details->compatible_with ( * other.m_type_details );
}

std::ostream & operator << ( std::ostream & os, const type_details & arg ) {
	return os << ( * arg.m_type_details );
}

std::strong_ordering type_details::operator <=> ( const type_details & other ) const {
	return * m_type_details <=> * other.m_type_details;
}

bool type_details::operator == ( const type_details & other ) const {
	return * m_type_details == * other.m_type_details;
}

std::unique_ptr < type_details_base > type_details_retriever < unsigned >::get ( ) {
	return std::make_unique < type_details_type > ( "unsigned" );
}

std::unique_ptr < type_details_base > type_details_retriever < long unsigned >::get ( ) {
	if constexpr ( std::is_same_v < long unsigned, size_t > ) {
		core::unique_ptr_set < type_details_base > types;
		types.insert ( std::make_unique < type_details_type > ( "long unsigned" ) );
		types.insert ( std::make_unique < type_details_type > ( "size_t" ) );
		return std::make_unique < type_details_variant_type > ( std::move ( types ) );
	} else {
		return std::make_unique < type_details_type > ( "long unsigned" );
	}
}

std::unique_ptr < type_details_base > type_details_retriever < int >::get ( ) {
	return std::make_unique < type_details_type > ( "int" );
}

std::unique_ptr < type_details_base > type_details_retriever < long >::get ( ) {
	return std::make_unique < type_details_type > ( "long" );
}

std::unique_ptr < type_details_base > type_details_retriever < char >::get ( ) {
	return std::make_unique < type_details_type > ( "char" );
}

std::unique_ptr < type_details_base > type_details_retriever < bool >::get ( ) {
	return std::make_unique < type_details_type > ( "bool" );
}

std::unique_ptr < type_details_base > type_details_retriever < double >::get ( ) {
	return std::make_unique < type_details_type > ( "double" );
}

std::unique_ptr < type_details_base > type_details_retriever < std::string >::get ( ) {
	return std::make_unique < type_details_type > ( "std::string" );
}

std::unique_ptr < type_details_base > type_details_retriever < ext::ostream >::get ( ) {
	return std::make_unique < type_details_type > ( "ext::ostream" );
}

} /* namespace core */
